@@ -54,15 +54,6 @@ class NoUniqueMatch(Exception):
     pass
 
 
-class AuthSystemNotFound(Exception):
-    """When the user specify a AuthSystem but not installed."""
-    def __init__(self, auth_system):
-        self.auth_system = auth_system
-
-    def __str__(self):
-        return "AuthSystemNotFound: %s" % repr(self.auth_system)
-
-
 class NoTokenLookupException(Exception):
     """This form of authentication does not support looking up
        endpoints from an existing token.
