@@ -23,7 +23,6 @@ import argparse
 import getpass
 import logging
 import sys
-import warnings
 
 from keystoneauth1 import loading
 from oslo_utils import encodeutils
@@ -41,7 +40,6 @@ except ImportError:
 
 import novaclient
 from novaclient import api_versions
-import novaclient.auth_plugin
 from novaclient import client
 from novaclient import exceptions as exc
 import novaclient.extension
@@ -233,7 +231,6 @@ class SecretsHelper(object):
             self.client.endpoint_type,
             self.client.service_type,
             self.client.service_name,
-            self.client.volume_service_name,
         ]
         for (index, key) in enumerate(keys):
             if key is None:
@@ -455,12 +452,6 @@ class OpenStackComputeShell(object):
             default=utils.env('OS_REGION_NAME', 'NOVA_REGION_NAME'),
             help=_('Defaults to env[OS_REGION_NAME].'))
 
-        parser.add_argument(
-            '--os-auth-system',
-            metavar='<auth-system>',
-            default=utils.env('OS_AUTH_SYSTEM'),
-            help=argparse.SUPPRESS)
-
         parser.add_argument(
             '--service-type',
             metavar='<service-type>',
@@ -502,15 +493,21 @@ class OpenStackComputeShell(object):
                    '"X.latest", defaults to env[OS_COMPUTE_API_VERSION].'))
 
         parser.add_argument(
-            '--bypass-url',
+            '--endpoint-override',
             metavar='<bypass-url>',
-            dest='bypass_url',
-            default=utils.env('NOVACLIENT_BYPASS_URL'),
+            dest='endpoint_override',
+            default=utils.env('NOVACLIENT_ENDPOINT_OVERRIDE',
+                              'NOVACLIENT_BYPASS_URL'),
             help=_("Use this API endpoint instead of the Service Catalog. "
-                   "Defaults to env[NOVACLIENT_BYPASS_URL]."))
+                   "Defaults to env[NOVACLIENT_ENDPOINT_OVERRIDE]."))
 
-        # The auth-system-plugins might require some extra options
-        novaclient.auth_plugin.load_auth_system_opts(parser)
+        parser.add_argument(
+            '--bypass-url',
+            action=DeprecatedAction,
+            use=_('use "%s"; this option will be removed after Pike OpenStack '
+                  'release.') % '--os-endpoint-override',
+            dest='endpoint_override',
+            help=argparse.SUPPRESS)
 
         self._append_global_identity_args(parser, argv)
 
@@ -632,16 +629,12 @@ class OpenStackComputeShell(object):
 
         self.setup_debugging(args.debug)
         self.extensions = []
-        do_help = ('help' in argv) or (
-            '--help' in argv) or ('-h' in argv) or not argv
+        do_help = args.help or not args_list or args_list[0] == 'help'
 
         # bash-completion should not require authentication
         skip_auth = do_help or (
             'bash-completion' in argv)
 
-        # Discover available auth plugins
-        novaclient.auth_plugin.discover_auth_systems()
-
         if not args.os_compute_api_version:
             api_version = api_versions.get_api_version(
                 DEFAULT_MAJOR_OS_COMPUTE_API_VERSION)
@@ -658,7 +651,6 @@ class OpenStackComputeShell(object):
             args, 'os_project_id', getattr(args, 'os_tenant_id', None))
         os_auth_url = args.os_auth_url
         os_region_name = args.os_region_name
-        os_auth_system = args.os_auth_system
 
         if "v2.0" not in os_auth_url:
             # NOTE(andreykurilin): assume that keystone V3 is used and try to
@@ -669,14 +661,19 @@ class OpenStackComputeShell(object):
             if not args.os_user_domain_id and not args.os_user_domain_name:
                 setattr(args, "os_user_domain_id", "default")
 
+        os_project_domain_id = args.os_project_domain_id
+        os_project_domain_name = args.os_project_domain_name
+        os_user_domain_id = args.os_project_domain_id
+        os_user_domain_name = args.os_project_domain_name
+
         endpoint_type = args.endpoint_type
         insecure = args.insecure
         service_type = args.service_type
         service_name = args.service_name
-        volume_service_name = args.volume_service_name
-        bypass_url = args.bypass_url
+        endpoint_override = args.endpoint_override
         os_cache = args.os_cache
         cacert = args.os_cacert
+        cert = args.os_cert
         timeout = args.timeout
 
         keystone_session = None
@@ -689,16 +686,6 @@ class OpenStackComputeShell(object):
         # Note if we don't auth we probably don't have a tenant ID so we can't
         # cache the token.
         auth_token = getattr(args, 'os_token', None)
-        management_url = bypass_url if bypass_url else None
-
-        if os_auth_system and os_auth_system != "keystone":
-            warnings.warn(_(
-                'novaclient auth plugins that are not keystone are deprecated.'
-                ' Auth plugins should now be done as plugins to keystoneauth'
-                ' and selected with --os-auth-type or OS_AUTH_TYPE'))
-            auth_plugin = novaclient.auth_plugin.load_plugin(os_auth_system)
-        else:
-            auth_plugin = None
 
         if not endpoint_type:
             endpoint_type = DEFAULT_NOVA_ENDPOINT_TYPE
@@ -713,29 +700,20 @@ class OpenStackComputeShell(object):
             # service type specified, we use default nova service type.
             service_type = DEFAULT_NOVA_SERVICE_TYPE
 
-        # If we have an auth token but no management_url, we must auth anyway.
+        # We should always auth unless we have a token and we're passing a
+        # specific endpoint
         # Expired tokens are handled by client.py:_cs_request
-        must_auth = not (auth_token and management_url)
-
-        # Do not use Keystone session for cases with no session support. The
-        # presence of auth_plugin means os_auth_system is present and is not
-        # keystone.
-        use_session = True
-        if auth_plugin or bypass_url or os_cache or volume_service_name:
-            use_session = False
+        must_auth = not (auth_token and endpoint_override)
 
         # FIXME(usrleon): Here should be restrict for project id same as
         # for os_username or os_password but for compatibility it is not.
         if must_auth and not skip_auth:
-            if auth_plugin:
-                auth_plugin.parse_opts(args)
 
-            if not auth_plugin or not auth_plugin.opts:
-                if not os_username and not os_user_id:
-                    raise exc.CommandError(
-                        _("You must provide a username "
-                          "or user ID via --os-username, --os-user-id, "
-                          "env[OS_USERNAME] or env[OS_USER_ID]"))
+            if not os_username and not os_user_id:
+                raise exc.CommandError(
+                    _("You must provide a username "
+                      "or user ID via --os-username, --os-user-id, "
+                      "env[OS_USERNAME] or env[OS_USER_ID]"))
 
             if not any([os_project_name, os_project_id]):
                 raise exc.CommandError(_("You must provide a project name or"
@@ -746,28 +724,16 @@ class OpenStackComputeShell(object):
                                          " interchangeably."))
 
             if not os_auth_url:
-                if os_auth_system and os_auth_system != 'keystone':
-                    os_auth_url = auth_plugin.get_auth_url()
+                raise exc.CommandError(
+                    _("You must provide an auth url "
+                      "via either --os-auth-url or env[OS_AUTH_URL]."))
 
-            if not os_auth_url:
-                    raise exc.CommandError(
-                        _("You must provide an auth url "
-                          "via either --os-auth-url or env[OS_AUTH_URL] "
-                          "or specify an auth_system which defines a "
-                          "default url with --os-auth-system "
-                          "or env[OS_AUTH_SYSTEM]"))
-
-            if use_session:
-                # Not using Nova auth plugin, so use keystone
-                with utils.record_time(self.times, args.timings,
-                                       'auth_url', args.os_auth_url):
-                    keystone_session = (
-                        loading.load_session_from_argparse_arguments(args))
-                    keystone_auth = (
-                        loading.load_auth_from_argparse_arguments(args))
-            else:
-                # set password for auth plugins
-                os_password = args.os_password
+            with utils.record_time(self.times, args.timings,
+                                   'auth_url', args.os_auth_url):
+                keystone_session = (
+                    loading.load_session_from_argparse_arguments(args))
+                keystone_auth = (
+                    loading.load_auth_from_argparse_arguments(args))
 
         if (not skip_auth and
                 not any([os_project_name, os_project_id])):
@@ -787,19 +753,21 @@ class OpenStackComputeShell(object):
         # microversion, so we just pass version 2 at here.
         self.cs = client.Client(
             api_versions.APIVersion("2.0"),
-            os_username, os_password, os_project_name,
-            tenant_id=os_project_id, user_id=os_user_id,
+            os_username, os_password, project_id=os_project_id,
+            project_name=os_project_name, user_id=os_user_id,
             auth_url=os_auth_url, insecure=insecure,
             region_name=os_region_name, endpoint_type=endpoint_type,
             extensions=self.extensions, service_type=service_type,
-            service_name=service_name, auth_system=os_auth_system,
-            auth_plugin=auth_plugin, auth_token=auth_token,
-            volume_service_name=volume_service_name,
-            timings=args.timings, bypass_url=bypass_url,
+            service_name=service_name, auth_token=auth_token,
+            timings=args.timings, endpoint_override=endpoint_override,
             os_cache=os_cache, http_log_debug=args.debug,
-            cacert=cacert, timeout=timeout,
+            cacert=cacert, cert=cert, timeout=timeout,
             session=keystone_session, auth=keystone_auth,
-            logger=self.client_logger)
+            logger=self.client_logger,
+            project_domain_id=os_project_domain_id,
+            project_domain_name=os_project_domain_name,
+            user_domain_id=os_user_domain_id,
+            user_domain_name=os_user_domain_name)
 
         if not skip_auth:
             if not api_version.is_latest():
@@ -857,55 +825,37 @@ class OpenStackComputeShell(object):
             auth_url=os_auth_url, insecure=insecure,
             region_name=os_region_name, endpoint_type=endpoint_type,
             extensions=self.extensions, service_type=service_type,
-            service_name=service_name, auth_system=os_auth_system,
-            auth_plugin=auth_plugin, auth_token=auth_token,
-            volume_service_name=volume_service_name,
-            timings=args.timings, bypass_url=bypass_url,
+            service_name=service_name, auth_token=auth_token,
+            timings=args.timings, endpoint_override=endpoint_override,
             os_cache=os_cache, http_log_debug=args.debug,
-            cacert=cacert, timeout=timeout,
-            session=keystone_session, auth=keystone_auth)
+            cacert=cacert, cert=cert, timeout=timeout,
+            session=keystone_session, auth=keystone_auth,
+            project_domain_id=os_project_domain_id,
+            project_domain_name=os_project_domain_name,
+            user_domain_id=os_user_domain_id,
+            user_domain_name=os_user_domain_name)
 
         # Now check for the password/token of which pieces of the
         # identifying keyring key can come from the underlying client
         if must_auth:
             helper = SecretsHelper(args, self.cs.client)
             self.cs.client.keyring_saver = helper
-            if (auth_plugin and auth_plugin.opts and
-                    "os_password" not in auth_plugin.opts):
-                use_pw = False
-            else:
-                use_pw = True
 
             tenant_id = helper.tenant_id
             # Allow commandline to override cache
             if not auth_token:
                 auth_token = helper.auth_token
-            if not management_url:
-                management_url = helper.management_url
-            if tenant_id and auth_token and management_url:
+            endpoint_override = endpoint_override or helper.management_url
+            if tenant_id and auth_token and endpoint_override:
                 self.cs.client.tenant_id = tenant_id
                 self.cs.client.auth_token = auth_token
-                self.cs.client.management_url = management_url
+                self.cs.client.management_url = endpoint_override
                 self.cs.client.password_func = lambda: helper.password
-            elif use_pw:
+            else:
                 # We're missing something, so auth with user/pass and save
                 # the result in our helper.
                 self.cs.client.password = helper.password
 
-        try:
-            # This does a couple of bits which are useful even if we've
-            # got the token + service URL already. It exits fast in that case.
-            if not utils.isunauthenticated(args.func):
-                if not use_session:
-                    # Only call authenticate() if Nova auth plugin is used.
-                    # If keystone is used, authentication is handled as part
-                    # of session.
-                    self.cs.authenticate()
-        except exc.Unauthorized:
-            raise exc.CommandError(_("Invalid OpenStack Nova credentials."))
-        except exc.AuthorizationFailure:
-            raise exc.CommandError(_("Unable to authorize user"))
-
         args.func(self.cs, args)
 
         if args.timings:
@@ -982,9 +932,13 @@ def main():
         OpenStackComputeShell().main(argv)
     except Exception as exc:
         logger.debug(exc, exc_info=1)
-        print(_("ERROR (%(type)s): %(msg)s") % {
+        if six.PY2:
+            message = encodeutils.safe_encode(six.text_type(exc))
+        else:
+            message = encodeutils.exception_to_unicode(exc)
+        print("ERROR (%(type)s): %(msg)s" % {
               'type': exc.__class__.__name__,
-              'msg': encodeutils.exception_to_unicode(exc)},
+              'msg': message},
               file=sys.stderr)
         sys.exit(1)
     except KeyboardInterrupt:
